@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2000-2003 Marco Pesenti Gritti
  *
@@ -13,9 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: popup-commands.c,v 1.65 2006/12/04 22:10:40 chpe Exp $
+ *  $Id: popup-commands.c 7030 2007-05-18 17:04:53Z diegoe $
  */
 
 #include "config.h"
@@ -178,10 +179,12 @@ save_property_url_completed_cb (EphyEmbedPersist *persist)
 
 		user_time = ephy_embed_persist_get_user_time (persist);
 		dest = ephy_embed_persist_get_dest (persist);
+
 		g_return_if_fail (dest != NULL);
 
-		eel_gconf_get_string (CONF_STATE_DOWNLOAD_DIR);
+		/* If save location is the desktop, nautilus will not open */
 		ephy_file_browse_to (dest, user_time);
+
 	}
 }
 
@@ -284,7 +287,13 @@ background_download_completed (EphyEmbedPersist *persist)
 	g_object_unref (persist);
 
 	/* open the "Background Properties" capplet */
-	ephy_file_launch_desktop_file ("background.desktop", bg, user_time);
+	if (!ephy_file_launch_desktop_file ("background.desktop", bg, user_time))
+	{
+		/* If the above try didn't work, then we try the Fedora name.
+		 * This is a fix for #387206, but is actually a workaround for
+		 * bugzilla.redhat.com #201867 */
+		ephy_file_launch_desktop_file ("gnome-background.desktop", bg, user_time);
+	}
 }
 
 void
@@ -361,22 +370,31 @@ popup_cmd_open_frame (GtkAction *action,
 	g_free (location);
 }
 
+/* Opens an image URI using its associated handler. Or, if that
+ * doesn't work, fallback to open the URI in a new browser window.
+ */
 static void
-image_open_uri (const char *address,
-		gboolean delete,
+image_open_uri (const char *remote_address,
+                const char *local_address,
 		guint32 user_time)
 {
 	gboolean success;
 
-	success = ephy_file_launch_handler (NULL, address, user_time);
+	success = ephy_file_launch_handler (NULL, local_address, user_time);
 
-	if (delete && success)
+	if (!success)
 	{
-		ephy_file_delete_on_exit (address);
+		ephy_shell_new_tab (ephy_shell, NULL, NULL, remote_address,
+				    EPHY_NEW_TAB_OPEN_PAGE |
+				    EPHY_NEW_TAB_IN_NEW_WINDOW);
 	}
-	else if (delete)
+
+	if (strcmp (remote_address, local_address) != 0)
 	{
-		gnome_vfs_unlink (address);
+		if (success)
+			ephy_file_delete_on_exit (local_address);
+		else
+			gnome_vfs_unlink (local_address);
 	}
 }
 
@@ -384,13 +402,15 @@ static void
 save_source_completed_cb (EphyEmbedPersist *persist)
 {
 	const char *dest;
+	const char *source;
 	guint32 user_time;
 
 	user_time = ephy_embed_persist_get_user_time (persist);
 	dest = ephy_embed_persist_get_dest (persist);
+	source = ephy_embed_persist_get_source (persist);
 	g_return_if_fail (dest != NULL);
 
-	image_open_uri (dest, TRUE, user_time);
+	image_open_uri (source, dest, user_time);
 }
 
 static void
@@ -455,7 +475,7 @@ popup_cmd_open_image (GtkAction *action,
 
 	if (strcmp (scheme, "file") == 0)
 	{
-		image_open_uri (address, FALSE,
+		image_open_uri (address, address,
 				gtk_get_current_event_time ());
 	}
 	else
